#!/bin/bash

# Installation automatisée du pack config de Alien2924
# Ce script a été ecrit par micky979 (michael@vdrbox.lautre.net)
# Divers correctifs par alien2924 (alien.film@gmail.com)

# Historique
# ==================================================================================
# v0.0.1 (13.09.2008) / micky979 :
#			Revision initiale
# v0.0.2 (14.09.2008) / micky979 :
#			Correction de rc$INIT en rc$INIT.d
#			Placement des scripts de démarrage dans init 2 3 4 5
#			Correction if de Etre root
#			Correction if de mime.types
#			Rajout d'une contrainte dans la recherche de ISODIR
# v0.0.3 (14.09.2008) / micky979 :
#			Rajout des sous répertoires dans /video
#			Correction du chemin usr/src/
# v0.0.4 (14.09.2008) / alien2924
#			Ajout du sous répertoire /tmp/vtx
#			Ajout de la création du fichier mime.types s'il n'existe pas
#			Correction pour la copie des fichiers dans /etc/sysconfig
#			Suppression de la création du répertoire /video/iso car fait par détection de la variable ISODIR
#			Suppression du no_remove dans /video/iso inutile
# v0.0.5 (14.09.2008) / micky979
#			Test si /etc/sysconfig/ existe, copie des fichiers de variables
#			Test si /etc/default/ existe, copie des fichiers de variables
#			Si ni /etc/sysconfig/, ni /etc/default existe, copie des fichiers de variables dans /etc/default-vdr/
# v0.0.6 (15.09.2008) / micky979
#			Reconfiguration de vdr-setup.fr_FR.xml suivant si c'est sysconfig, default/vdr ou default-vdr
# v0.0.7 (20.09.2008) / micky979
#			Correction de la variable BURN
#			Correction du chemin vers vdr-setup.fr_FR.xml
#			Force les liens symboliques dans /etc/rc*.d/

PATH=/usr/sbin:/usr/bin:/sbin:/bin

# Etre root
if [ $(whoami) != "root" ]
then
	echo 'Vous devez être super-utilisateur pour exécuter ce script !'
	exit 1
fi

# Variables pour script et mime.types
REP_SCRIPT=$(dirname $0)
MIME="/etc/mime.types"

# Distribution Debian ou autres
if [ -e "/etc/debian_version" ]
then
	DISTRIBUTION="DEBIAN"
else
	DISTRIBUTION="AUTRES"
fi

# Création des répertoires
mkdir -p /usr/local/lib/vdr
mkdir -p /usr/local/man/man1
mkdir -p /usr/local/man/man5

# Création de l'arborescence /video
mkdir -p /video/divx/webvideo
mkdir -p /video/images
mkdir -p /video/mp3
mkdir -p /video/records
mkdir -p /video/themes
mkdir -p /video/tmp
mkdir -p /video/tmp_vdrrip
mkdir -p /video/vdrconfdev
mkdir -p /video/.vdrconvert

# Copie des fichiers
cp $REP_SCRIPT/fichiers_conf_racine/etc/init.d/* /etc/init.d/
cp $REP_SCRIPT/fichiers_conf_racine/usr/local/bin/* /usr/local/bin/
cp $REP_SCRIPT/fichiers_conf_racine/usr/src/dvb-cwidx-kernel-2.6.22.diff /usr/src/
cp $REP_SCRIPT/fichiers_conf_racine/video/.vdrconvert/vdrconvert.env /video/.vdrconvert

# Copie du contenu de fichiers_conf_racine/etc/sysconfig/ et reconfiguration de vdr-setup.fr_FR.xml
if [ -d  "/etc/sysconfig/" ]
then
	cp $REP_SCRIPT/fichiers_conf_racine/etc/sysconfig/* /etc/sysconfig/
else
	if [ -d "/etc/default/" ]
	then
		mkdir -p /etc/default/vdr
		cp $REP_SCRIPT/fichiers_conf_racine/etc/sysconfig/* /etc/default/vdr/
		sed -i s/'\/etc\/sysconfig'/'\/etc\/default\/vdr'/g $REP_SCRIPT/plugins/setup/vdr-setup.fr_FR.xml
	else
		mkdir -p /etc/default-vdr
		cp cp $REP_SCRIPT/fichiers_conf_racine/etc/sysconfig/* /etc/default-vdr/
		echo "Copie des fichiers de variables dans /etc/default-vdr/"
		sed -i s/'\/etc\/sysconfig'/'\/etc\/default-vdr'/g $REP_SCRIPT/plugins/setup/vdr-setup.fr_FR.xml
	fi
fi

# Création du sous répertoire vtx dans /tmp (Pour le plugin Osdteletext)
mkdir -p /tmp/vtx

# Création répertoire (Pour le plugin Burn)
BURN=$(grep "^ISODIR=" /usr/local/src/VDR/Make.config 2>/dev/null | awk -F"=" '{print $2}' 2>/dev/null)
if [ "$BURN" != "" ]
then
	mkdir -p $BURN
else
	echo "Variable ISODIR (pour Burn) non définie dans le Make.config de VDR, création du répertoire par défaut /pub/export/"
	mkdir -p /pub/export
fi

# Fichier mime.types (Pour le plugin Webvideo)
if [ -e $MIME ]
then
	grep 'video/flv' $MIME > /dev/null 2>&1
	RETOUR=$?
	if [ "$RETOUR" != "0" ]
	then
		echo "video/flv	flv" >> $MIME
	fi
	grep 'video/x-flv' $MIME > /dev/null 2>&1
	RETOUR=$?
	if [ "$RETOUR" != "0" ]
	then
		echo "video/x-flv	flv" >> $MIME
	fi
else
	echo "video/flv	flv" > $MIME
	echo "video/x-flv	flv" >> $MIME
fi

# Création fichier de log
touch /var/log/vdr.log
chmod 777 /var/log/vdr.log

# Démarrage et arrêt automatique de VDR pour les différents runlevel du SystemV
if [ "$DISTRIBUTION" = "AUTRES" ]
then
	# Démarrage
	for INIT in 2 3 4 5
	do
		ln -fns /etc/init.d/dvb /etc/rc$INIT.d/S20dvb
		ln -fns /etc/init.d/vdrlaunch /etc/rc$INIT.d/S97vdrlaunch
		ln -fns /etc/init.d/vdradmind /etc/rc$INIT.d/S98vdradmind
	done
	# Arrêt
	for INIT in 0 1 6
	do
		ln -fns /etc/init.d/vdradmind /etc/rc$INIT.d/K01vdradmind
		ln -fns /etc/init.d/vdrlaunch /etc/rc$INIT.d/K02vdrlaunch
	done
else
	if [ "$DISTRIBUTION" = "DEBIAN" ]
	then
		update-rc.d dvb defaults
		update-rc.d vdrlaunch start 97 2 3 4 5 . stop 02 0 1 6 .
		update-rc.d vdradmind start 98 2 3 4 5 . stop 01 0 1 6 .
	else
		echo "Distribution inconnue"
	fi
fi
